@@ -2,75 +2,71 @@ require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
 describe "Database schema parser" do
   before do
-    @iom = INTEGRATION_DB.identifier_output_method
-    @iim = INTEGRATION_DB.identifier_input_method
-    @defsch = INTEGRATION_DB.default_schema
-    @qi = INTEGRATION_DB.quote_identifiers?
+    @iom = DB.identifier_output_method
+    @iim = DB.identifier_input_method
+    @qi = DB.quote_identifiers?
   end
   after do
-    INTEGRATION_DB.identifier_output_method = @iom
-    INTEGRATION_DB.identifier_input_method = @iim
-    INTEGRATION_DB.default_schema = @defsch
-    INTEGRATION_DB.quote_identifiers = @qi
-    INTEGRATION_DB.drop_table?(:items)
+    DB.identifier_output_method = @iom
+    DB.identifier_input_method = @iim
+    DB.quote_identifiers = @qi
+    DB.drop_table?(:items)
   end
 
   specify "should handle a database with a identifier methods" do
-    INTEGRATION_DB.identifier_output_method = :reverse
-    INTEGRATION_DB.identifier_input_method = :reverse
-    INTEGRATION_DB.quote_identifiers = true
-    INTEGRATION_DB.default_schema = nil if INTEGRATION_DB.default_schema
-    INTEGRATION_DB.create_table!(:items){Integer :number}
+    DB.identifier_output_method = :reverse
+    DB.identifier_input_method = :reverse
+    DB.quote_identifiers = true
+    DB.create_table!(:items){Integer :number}
     begin
-      INTEGRATION_DB.schema(:items, :reload=>true).should be_a_kind_of(Array)
-      INTEGRATION_DB.schema(:items, :reload=>true).first.first.should == :number
+      DB.schema(:items, :reload=>true).should be_a_kind_of(Array)
+      DB.schema(:items, :reload=>true).first.first.should == :number
     ensure 
-      INTEGRATION_DB.drop_table(:items)
+      DB.drop_table(:items)
     end
   end
 
   specify "should handle a dataset with identifier methods different than the database's" do
-    INTEGRATION_DB.identifier_output_method = :reverse
-    INTEGRATION_DB.identifier_input_method = :reverse
-    INTEGRATION_DB.quote_identifiers = true
-    INTEGRATION_DB.default_schema = nil if INTEGRATION_DB.default_schema
-    INTEGRATION_DB.create_table!(:items){Integer :number}
-    INTEGRATION_DB.identifier_output_method = @iom
-    INTEGRATION_DB.identifier_input_method = @iim
-    ds = INTEGRATION_DB[:items]
+    DB.identifier_output_method = :reverse
+    DB.identifier_input_method = :reverse
+    DB.quote_identifiers = true
+    DB.create_table!(:items){Integer :number}
+    DB.identifier_output_method = @iom
+    DB.identifier_input_method = @iim
+    ds = DB[:items]
     ds.identifier_output_method = :reverse
     ds.identifier_input_method = :reverse
     begin
-      INTEGRATION_DB.schema(ds, :reload=>true).should be_a_kind_of(Array)
-      INTEGRATION_DB.schema(ds, :reload=>true).first.first.should == :number
+      DB.schema(ds, :reload=>true).should be_a_kind_of(Array)
+      DB.schema(ds, :reload=>true).first.first.should == :number
     ensure 
-      INTEGRATION_DB.identifier_output_method = :reverse
-      INTEGRATION_DB.identifier_input_method = :reverse
-      INTEGRATION_DB.drop_table(:items)
+      DB.identifier_output_method = :reverse
+      DB.identifier_input_method = :reverse
+      DB.drop_table(:items)
     end
   end
 
   specify "should not issue an sql query if the schema has been loaded unless :reload is true" do
-    INTEGRATION_DB.create_table!(:items){Integer :number}
-    INTEGRATION_DB.schema(:items, :reload=>true)
-    INTEGRATION_DB.schema(:items)
-    INTEGRATION_DB.schema(:items, :reload=>true)
+    DB.create_table!(:items){Integer :number}
+    DB.schema(:items, :reload=>true)
+    DB.schema(:items)
+    DB.schema(:items, :reload=>true)
   end
 
   specify "Model schema should include columns in the table, even if they aren't selected" do
-    INTEGRATION_DB.create_table!(:items){String :a; Integer :number}
-    m = Sequel::Model(INTEGRATION_DB[:items].select(:a))
+    DB.create_table!(:items){String :a; Integer :number}
+    m = Sequel::Model(DB[:items].select(:a))
     m.columns.should == [:a]
     m.db_schema[:number][:type].should == :integer
   end
 
   specify "should raise an error when the table doesn't exist" do
-    proc{INTEGRATION_DB.schema(:no_table)}.should raise_error(Sequel::Error)
+    proc{DB.schema(:no_table)}.should raise_error(Sequel::Error)
   end
 
   specify "should return the schema correctly" do
-    INTEGRATION_DB.create_table!(:items){Integer :number}
-    schema = INTEGRATION_DB.schema(:items, :reload=>true)
+    DB.create_table!(:items){Integer :number}
+    schema = DB.schema(:items, :reload=>true)
     schema.should be_a_kind_of(Array)
     schema.length.should == 1
     col = schema.first
@@ -80,128 +76,122 @@ describe "Database schema parser" do
     col_info = col.last
     col_info.should be_a_kind_of(Hash)
     col_info[:type].should == :integer
-    INTEGRATION_DB.schema(:items)
+    DB.schema(:items)
   end
 
   specify "should parse primary keys from the schema properly" do
-    INTEGRATION_DB.create_table!(:items){Integer :number}
-    INTEGRATION_DB.schema(:items).collect{|k,v| k if v[:primary_key]}.compact.should == []
-    INTEGRATION_DB.create_table!(:items){primary_key :number}
-    INTEGRATION_DB.schema(:items).collect{|k,v| k if v[:primary_key]}.compact.should == [:number]
-    INTEGRATION_DB.create_table!(:items){Integer :number1; Integer :number2; primary_key [:number1, :number2]}
-    INTEGRATION_DB.schema(:items).collect{|k,v| k if v[:primary_key]}.compact.should == [:number1, :number2]
+    DB.create_table!(:items){Integer :number}
+    DB.schema(:items).collect{|k,v| k if v[:primary_key]}.compact.should == []
+    DB.create_table!(:items){primary_key :number}
+    DB.schema(:items).collect{|k,v| k if v[:primary_key]}.compact.should == [:number]
+    DB.create_table!(:items){Integer :number1; Integer :number2; primary_key [:number1, :number2]}
+    DB.schema(:items).collect{|k,v| k if v[:primary_key]}.compact.should == [:number1, :number2]
   end
 
   specify "should parse NULL/NOT NULL from the schema properly" do
-    INTEGRATION_DB.create_table!(:items){Integer :number, :null=>true}
-    INTEGRATION_DB.schema(:items).first.last[:allow_null].should == true
-    INTEGRATION_DB.create_table!(:items){Integer :number, :null=>false}
-    INTEGRATION_DB.schema(:items).first.last[:allow_null].should == false
+    DB.create_table!(:items){Integer :number, :null=>true}
+    DB.schema(:items).first.last[:allow_null].should == true
+    DB.create_table!(:items){Integer :number, :null=>false}
+    DB.schema(:items).first.last[:allow_null].should == false
   end
 
   specify "should parse defaults from the schema properly" do
-    INTEGRATION_DB.create_table!(:items){Integer :number}
-    INTEGRATION_DB.schema(:items).first.last[:ruby_default].should == nil
-    INTEGRATION_DB.create_table!(:items){Integer :number, :default=>0}
-    INTEGRATION_DB.schema(:items).first.last[:ruby_default].should == 0
-    INTEGRATION_DB.create_table!(:items){String :a, :default=>"blah"}
-    INTEGRATION_DB.schema(:items).first.last[:ruby_default].should == 'blah'
+    DB.create_table!(:items){Integer :number}
+    DB.schema(:items).first.last[:ruby_default].should == nil
+    DB.create_table!(:items){Integer :number, :default=>0}
+    DB.schema(:items).first.last[:ruby_default].should == 0
+    DB.create_table!(:items){String :a, :default=>"blah"}
+    DB.schema(:items).first.last[:ruby_default].should == 'blah'
   end
 
   specify "should parse current timestamp defaults from the schema properly" do
-    INTEGRATION_DB.create_table!(:items){Time :a, :default=>Sequel::CURRENT_TIMESTAMP}
-    INTEGRATION_DB.schema(:items).first.last[:ruby_default].should == Sequel::CURRENT_TIMESTAMP
+    DB.create_table!(:items){Time :a, :default=>Sequel::CURRENT_TIMESTAMP}
+    DB.schema(:items).first.last[:ruby_default].should == Sequel::CURRENT_TIMESTAMP
   end
 
   cspecify "should parse current date defaults from the schema properly", :mysql, :oracle do
-    INTEGRATION_DB.create_table!(:items){Date :a, :default=>Sequel::CURRENT_DATE}
-    INTEGRATION_DB.schema(:items).first.last[:ruby_default].should == Sequel::CURRENT_DATE
+    DB.create_table!(:items){Date :a, :default=>Sequel::CURRENT_DATE}
+    DB.schema(:items).first.last[:ruby_default].should == Sequel::CURRENT_DATE
   end
 
   cspecify "should parse types from the schema properly", [:jdbc, :db2], :oracle do
-    INTEGRATION_DB.create_table!(:items){Integer :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :integer
-    INTEGRATION_DB.create_table!(:items){Fixnum :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :integer
-    INTEGRATION_DB.create_table!(:items){Bignum :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :integer
-    INTEGRATION_DB.create_table!(:items){Float :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :float
-    INTEGRATION_DB.create_table!(:items){BigDecimal :number, :size=>[11, 2]}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :decimal
-    INTEGRATION_DB.create_table!(:items){Numeric :number, :size=>[12, 0]}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :integer
-    INTEGRATION_DB.create_table!(:items){String :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :string
-    INTEGRATION_DB.create_table!(:items){Date :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :date
-    INTEGRATION_DB.create_table!(:items){Time :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :datetime
-    INTEGRATION_DB.create_table!(:items){DateTime :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :datetime
-    INTEGRATION_DB.create_table!(:items){File :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :blob
-    INTEGRATION_DB.create_table!(:items){TrueClass :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :boolean
-    INTEGRATION_DB.create_table!(:items){FalseClass :number}
-    INTEGRATION_DB.schema(:items).first.last[:type].should == :boolean
+    DB.create_table!(:items){Integer :number}
+    DB.schema(:items).first.last[:type].should == :integer
+    DB.create_table!(:items){Fixnum :number}
+    DB.schema(:items).first.last[:type].should == :integer
+    DB.create_table!(:items){Bignum :number}
+    DB.schema(:items).first.last[:type].should == :integer
+    DB.create_table!(:items){Float :number}
+    DB.schema(:items).first.last[:type].should == :float
+    DB.create_table!(:items){BigDecimal :number, :size=>[11, 2]}
+    DB.schema(:items).first.last[:type].should == :decimal
+    DB.create_table!(:items){Numeric :number, :size=>[12, 0]}
+    DB.schema(:items).first.last[:type].should == :integer
+    DB.create_table!(:items){String :number}
+    DB.schema(:items).first.last[:type].should == :string
+    DB.create_table!(:items){Date :number}
+    DB.schema(:items).first.last[:type].should == :date
+    DB.create_table!(:items){Time :number}
+    DB.schema(:items).first.last[:type].should == :datetime
+    DB.create_table!(:items){DateTime :number}
+    DB.schema(:items).first.last[:type].should == :datetime
+    DB.create_table!(:items){File :number}
+    DB.schema(:items).first.last[:type].should == :blob
+    DB.create_table!(:items){TrueClass :number}
+    DB.schema(:items).first.last[:type].should == :boolean
+    DB.create_table!(:items){FalseClass :number}
+    DB.schema(:items).first.last[:type].should == :boolean
   end
-end if INTEGRATION_DB.respond_to?(:schema_parse_table, true)
 
-test_indexes = begin
-  INTEGRATION_DB.drop_table?(:blah)
-  INTEGRATION_DB.indexes(:blah)
-  true
-rescue Sequel::NotImplemented
-  false
-rescue
-  true
-end
+  specify "should parse maximum length for string columns" do
+    DB.create_table!(:items){String :a, :size=>4}
+    DB.schema(:items).first.last[:max_length].should == 4
+    DB.create_table!(:items){String :a, :fixed=>true, :size=>3}
+    DB.schema(:items).first.last[:max_length].should == 3
+  end
+end if DB.supports_schema_parsing?
+
 describe "Database index parsing" do
   after do
-    INTEGRATION_DB.drop_table?(:items)
+    DB.drop_table?(:items)
   end
 
   specify "should parse indexes into a hash" do
     # Delete :deferrable entry, since not all adapters implement it
-    f = lambda{h = INTEGRATION_DB.indexes(:items); h.values.each{|h2| h2.delete(:deferrable)}; h}
+    f = lambda{h = DB.indexes(:items); h.values.each{|h2| h2.delete(:deferrable)}; h}
 
-    INTEGRATION_DB.create_table!(:items){Integer :n; Integer :a}
+    DB.create_table!(:items){Integer :n; Integer :a}
     f.call.should == {}
-    INTEGRATION_DB.add_index(:items, :n)
+    DB.add_index(:items, :n)
     f.call.should == {:items_n_index=>{:columns=>[:n], :unique=>false}}
-    INTEGRATION_DB.drop_index(:items, :n)
+    DB.drop_index(:items, :n)
     f.call.should == {}
-    INTEGRATION_DB.add_index(:items, :n, :unique=>true, :name=>:blah_blah_index)
+    DB.add_index(:items, :n, :unique=>true, :name=>:blah_blah_index)
     f.call.should == {:blah_blah_index=>{:columns=>[:n], :unique=>true}}
-    INTEGRATION_DB.add_index(:items, [:n, :a])
+    DB.add_index(:items, [:n, :a])
     f.call.should == {:blah_blah_index=>{:columns=>[:n], :unique=>true}, :items_n_a_index=>{:columns=>[:n, :a], :unique=>false}}
-    INTEGRATION_DB.drop_index(:items, :n, :name=>:blah_blah_index)
+    DB.drop_index(:items, :n, :name=>:blah_blah_index)
     f.call.should == {:items_n_a_index=>{:columns=>[:n, :a], :unique=>false}}
-    INTEGRATION_DB.drop_index(:items, [:n, :a])
+    DB.drop_index(:items, [:n, :a])
     f.call.should == {}
   end
   
   specify "should not include a primary key index" do
-    INTEGRATION_DB.create_table!(:items){primary_key :n}
-    INTEGRATION_DB.indexes(:items).should == {}
-    INTEGRATION_DB.create_table!(:items){Integer :n; Integer :a; primary_key [:n, :a]}
-    INTEGRATION_DB.indexes(:items).should == {}
+    DB.create_table!(:items){primary_key :n}
+    DB.indexes(:items).should == {}
+    DB.create_table!(:items){Integer :n; Integer :a; primary_key [:n, :a]}
+    DB.indexes(:items).should == {}
   end
-end if test_indexes
 
-test_foreign_key_list = begin
-  INTEGRATION_DB.drop_table?(:blah)
-  INTEGRATION_DB.foreign_key_list(:blah)
-  true
-rescue Sequel::NotImplemented
-  false
-rescue
-  true
-end
+  cspecify "should not include partial indexes", :sqlite do
+    DB.create_table!(:items){Integer :n; Integer :a; index :n, :where=>proc{n > 10}}
+    DB.indexes(:items).should == {}
+  end if DB.supports_partial_indexes?
+end if DB.supports_index_parsing?
+
 describe "Database foreign key parsing" do
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @pr = lambda do |table, *expected|
       actual = @db.foreign_key_list(table).sort_by{|c| c[:columns].map{|s| s.to_s}.join << (c[:key]||[]).map{|s| s.to_s}.join}.map{|v| v.values_at(:columns, :table, :key)}
       actual.zip(expected).each do |a, e|
@@ -223,7 +213,7 @@ describe "Database foreign key parsing" do
   end
 
   specify "should parse foreign key information into an array of hashes" do
-    @db.create_table!(:a, :engine=>:InnoDB){primary_key :c; Integer :d; index :d, :unique=>true}
+    @db.create_table!(:a, :engine=>:InnoDB){primary_key :c; Integer :d, :null => false, :unique => true}
     @db.create_table!(:b, :engine=>:InnoDB){foreign_key :e, :a}
     @pr[:a]
     @pr[:b, [[:e], :a, [:pk, :c]]]
@@ -234,21 +224,30 @@ describe "Database foreign key parsing" do
     @db.alter_table(:b){add_foreign_key [:f], :a, :key=>[:c]}
     @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:c]], [[:f], :a, [:d]]]
 
-    @db.alter_table(:a){add_index [:d, :c], :unique=>true}
+    @db.alter_table(:a){add_unique_constraint [:d, :c]}
     @db.alter_table(:b){add_foreign_key [:f, :e], :a, :key=>[:d, :c]}
     @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:c]], [[:f], :a, [:d]], [[:f, :e], :a, [:d, :c]]]
+
+    @db.alter_table(:b){drop_foreign_key [:f, :e]}
+    @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:c]], [[:f], :a, [:d]]]
+
+    @db.alter_table(:b){drop_foreign_key :e}
+    @pr[:b, [[:f], :a, [:c]], [[:f], :a, [:d]]]
+
+    proc{@db.alter_table(:b){drop_foreign_key :f}}.should raise_error(Sequel::Error)
+    @pr[:b, [[:f], :a, [:c]], [[:f], :a, [:d]]]
   end
 
   specify "should handle composite foreign and primary keys" do
-    @db.create_table!(:a, :engine=>:InnoDB){Integer :b; Integer :c; primary_key [:b, :c]; index [:c, :b], :unique=>true}
-    @db.create_table!(:b, :engine=>:InnoDB){Integer :e; Integer :f; foreign_key [:e, :f], :a; foreign_key [:f, :e], :a, :key=>[:c, :b]}
-    @pr[:b, [[:e, :f], :a, [:pk, :b, :c]], [[:f, :e], :a, [:c, :b]]]
+    @db.create_table!(:a, :engine=>:InnoDB){Integer :b, :null=>false; Integer :c, :null=>false; Integer :d, :null=>false; primary_key [:b, :c]; unique [:d, :c]}
+    @db.create_table!(:b, :engine=>:InnoDB){Integer :e, :null=>false; Integer :f, :null=>false; Integer :g, :null=>false; foreign_key [:e, :f], :a; foreign_key [:g, :f], :a, :key=>[:d, :c]}
+    @pr[:b, [[:e, :f], :a, [:pk, :b, :c]], [[:g, :f], :a, [:d, :c]]]
   end
-end if test_foreign_key_list
+end if DB.supports_foreign_key_parsing?
 
 describe "Database schema modifiers" do
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @ds = @db[:items]
   end
   after do
@@ -274,11 +273,81 @@ describe "Database schema modifiers" do
     @db[:items2].all.should == [{:number=>10}]
   end
   
+  specify "should not raise an error if table doesn't exist when using drop_table :if_exists" do
+    proc{@db.drop_table(:items, :if_exists=>true)}.should_not raise_error
+  end if DB.supports_drop_table_if_exists?
+
+  describe "views" do
+    before do
+      @db.drop_view(:items_view2) rescue nil
+      @db.drop_view(:items_view) rescue nil
+      @db.create_table(:items){Integer :number}
+      @ds.insert(:number=>1)
+      @ds.insert(:number=>2)
+    end
+    after do
+      @db.drop_view(:items_view2) rescue nil
+      @db.drop_view(:items_view) rescue nil
+    end
+
+    specify "should create views correctly" do
+      @db.create_view(:items_view, @ds.where(:number=>1))
+      @db[:items_view].map(:number).should == [1]
+    end
+
+    specify "should create views with check options correctly" do
+      @db.create_view(:items_view, @ds.where{number > 2}, :check=>true)
+      proc{@db[:items_view].insert(1)}.should raise_error(Sequel::DatabaseError)
+      @db[:items_view].insert(3)
+      @db[:items_view].select_order_map(:number).should == [3]
+      @db.create_view(:items_view2, @db[:items_view].where{number > 1}, :check=>true)
+      proc{@db[:items_view2].insert(1)}.should raise_error(Sequel::DatabaseError)
+      proc{@db[:items_view2].insert(2)}.should raise_error(Sequel::DatabaseError)
+      @db[:items_view2].insert(4)
+      @db[:items_view2].select_order_map(:number).should == [3, 4]
+      @ds.select_order_map(:number).should == [1, 2, 3, 4]
+    end if DB.supports_views_with_check_option?
+
+    specify "should create views with local check options correctly" do
+      @db.create_view(:items_view, @ds.where{number > 2})
+      @db[:items_view].insert(3)
+      @db[:items_view].select_order_map(:number).should == [3]
+      @db.create_view(:items_view2, @db[:items_view].where{number > 1}, :check=>:local)
+      proc{@db[:items_view2].insert(1)}.should raise_error(Sequel::DatabaseError)
+      @db[:items_view2].insert(2)
+      @db[:items_view2].insert(4)
+      @db[:items_view2].select_order_map(:number).should == [3, 4]
+      @ds.select_order_map(:number).should == [1, 2, 2, 3, 4]
+    end if DB.supports_views_with_local_check_option?
+
+    cspecify "should create views with explicit columns correctly", :sqlite do
+      @db.create_view(:items_view, @ds.where(:number=>1), :columns=>[:n])
+      @db[:items_view].map(:n).should == [1]
+    end
+
+    specify "should drop views correctly" do
+      @db.create_view(:items_view, @ds.where(:number=>1))
+      @db.drop_view(:items_view)
+      proc{@db[:items_view].map(:number)}.should raise_error(Sequel::DatabaseError)
+    end
+
+    specify "should not raise an error if view doesn't exist when using drop_view :if_exists" do
+      proc{@db.drop_view(:items_view, :if_exists=>true)}.should_not raise_error
+    end if DB.supports_drop_table_if_exists?
+
+    specify "should create or replace views correctly" do
+      @db.create_or_replace_view(:items_view, @ds.where(:number=>1))
+      @db[:items_view].map(:number).should == [1]
+      @db.create_or_replace_view(:items_view, @ds.where(:number=>2))
+      @db[:items_view].map(:number).should == [2]
+    end
+  end
+  
   specify "should handle create table in a rolled back transaction" do
     @db.drop_table?(:items)
     @db.transaction(:rollback=>:always){@db.create_table(:items){Integer :number}}
-    @db.table_exists?(:items).should be_false
-  end if INTEGRATION_DB.supports_transactional_ddl?
+    @db.table_exists?(:items).should == false
+  end if DB.supports_transactional_ddl?
   
   describe "join tables" do
     after do
@@ -296,9 +365,9 @@ describe "Database schema modifiers" do
   end
 
   specify "should create temporary tables without raising an exception" do
-    @db.create_table!(:items, :temp=>true){Integer :number}
+    @db.create_table!(:items_temp, :temp=>true){Integer :number}
   end
-  
+
   specify "should have create_table? only create the table if it doesn't already exist" do
     @db.create_table!(:items){String :a}
     @db.create_table?(:items){String :b}
@@ -336,6 +405,14 @@ describe "Database schema modifiers" do
     @ds.columns!.should == [:number]
   end
 
+  specify "should allow creating partial indexes with tables" do
+    @db.create_table!(:items){Integer :number; index :number, :where=>proc{number > 10}}
+    @db.table_exists?(:items).should == true
+    @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:number]
+    @ds.insert([10])
+    @ds.columns!.should == [:number]
+  end if DB.supports_partial_indexes?
+
   specify "should handle combination of default, unique, and not null" do
     @db.create_table!(:items){Integer :number, :default=>0, :null=>false, :unique=>true}
     @db.table_exists?(:items).should == true
@@ -343,6 +420,13 @@ describe "Database schema modifiers" do
     @ds.insert([10])
   end
 
+  specify "should be able to specify constraint names for column constraints" do
+    @db.create_table!(:items2){primary_key :id, :primary_key_constraint_name=>:foo_pk}
+    @db.create_table!(:items){foreign_key :id, :items2, :unique=>true, :foreign_key_constraint_name => :foo_fk, :unique_constraint_name => :foo_uk, :null=>false}
+    @db.alter_table(:items){drop_constraint :foo_fk, :type=>:foreign_key; drop_constraint :foo_uk, :type=>:unique}
+    @db.alter_table(:items2){drop_constraint :foo_pk, :type=>:primary_key}
+  end
+
   specify "should handle foreign keys correctly when creating tables" do
     @db.create_table!(:items) do 
       primary_key :id
@@ -364,7 +448,7 @@ describe "Database schema modifiers" do
     @ds.all.should == [{:number=>10, :name=>nil}]
   end
 
-  cspecify "should add primary key columns to tables correctly", :h2, :derby do
+  cspecify "should add primary key columns to tables correctly", :derby do
     @db.create_table!(:items){Integer :number}
     @ds.insert(:number=>10)
     @db.alter_table(:items){add_primary_key :id}
@@ -382,7 +466,7 @@ describe "Database schema modifiers" do
     proc{@ds.insert(10)}.should_not raise_error
   end
 
-  cspecify "should add foreign key columns to tables correctly", :hsqldb do
+  specify "should add foreign key columns to tables correctly" do
     @db.create_table!(:items){primary_key :id}
     @ds.insert
     i = @ds.get(:id)
@@ -509,7 +593,7 @@ describe "Database schema modifiers" do
   end
 
   specify "should add unnamed unique constraints and foreign key table constraints correctly" do
-    @db.create_table!(:items, :engine=>:InnoDB){Integer :id; Integer :item_id}
+    @db.create_table!(:items, :engine=>:InnoDB){Integer :id, :null => false; Integer :item_id, :null => false}
     @db.alter_table(:items) do
       add_unique_constraint [:item_id, :id]
       add_foreign_key [:id, :item_id], :items, :key=>[:item_id, :id]
@@ -571,18 +655,16 @@ describe "Database schema modifiers" do
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id]
   end
 
-  cspecify "should remove foreign key columns from tables correctly", :h2, :mssql, :hsqldb do
-    # MySQL with InnoDB cannot drop foreign key columns unless you know the
-    # name of the constraint, see Bug #14347
-    @db.create_table!(:items, :engine=>:MyISAM) do
+  specify "should remove foreign key columns from tables correctly" do
+    @db.create_table!(:items, :engine=>:InnoDB) do
       primary_key :id
       Integer :i
       foreign_key :item_id, :items
     end
     @ds.insert(:i=>10)
-    @db.drop_column(:items, :item_id)
+    @db.alter_table(:items){drop_foreign_key :item_id}
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id, :i]
-  end
+  end if DB.supports_foreign_key_parsing?
 
   specify "should remove multiple columns in a single alter_table block" do
     @db.create_table!(:items) do
@@ -628,7 +710,7 @@ describe "Database schema modifiers" do
     @db.create_table!(:items){foreign_key :id, :items2, :deferrable=>true}
     proc{@db[:items].insert(1)}.should raise_error(Sequel::DatabaseError)
     proc{@db.transaction{proc{@db[:items].insert(1)}.should_not raise_error}}.should raise_error(Sequel::DatabaseError)
-  end if INTEGRATION_DB.supports_deferrable_foreign_key_constraints?
+  end if DB.supports_deferrable_foreign_key_constraints?
 
   specify "should support deferrable unique constraints when creating or altering tables" do
     @db.create_table!(:items){Integer :t; unique [:t], :name=>:atest_def, :deferrable=>true, :using=>:btree}
@@ -643,15 +725,9 @@ describe "Database schema modifiers" do
     @db[:items].insert(2)
     proc{@db[:items].insert(2)}.should raise_error(Sequel::DatabaseError)
     proc{@db.transaction{proc{@db[:items].insert(2)}.should_not raise_error}}.should raise_error(Sequel::DatabaseError)
-  end if INTEGRATION_DB.supports_deferrable_constraints?
+  end if DB.supports_deferrable_constraints?
 end
 
-test_tables = begin
-  INTEGRATION_DB.tables
-  true
-rescue Sequel::NotImplemented
-  false
-end
 describe "Database#tables" do
   before do
     class ::String
@@ -660,7 +736,7 @@ describe "Database#tables" do
         "xxxxx#{@@xxxxx += 1}"
       end
     end
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table(:sequel_test_table){Integer :a}
     @db.create_view :sequel_test_view, @db[:sequel_test_table]
     @iom = @db.identifier_output_method
@@ -686,14 +762,8 @@ describe "Database#tables" do
     @db.identifier_input_method = :xxxxx
     @db.tables.each{|t| t.to_s.should =~ /\Ax{5}\d+\z/}
   end
-end if test_tables
+end if DB.supports_table_listing?
 
-test_views = begin
-  INTEGRATION_DB.views
-  true
-rescue Sequel::NotImplemented
-  false
-end
 describe "Database#views" do
   before do
     class ::String
@@ -702,7 +772,7 @@ describe "Database#views" do
         "xxxxx#{@@xxxxx += 1}"
       end
     end
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table(:sequel_test_table){Integer :a}
     @db.create_view :sequel_test_view, @db[:sequel_test_table]
     @iom = @db.identifier_output_method
@@ -728,4 +798,4 @@ describe "Database#views" do
     @db.identifier_input_method = :xxxxx
     @db.views.each{|t| t.to_s.should =~ /\Ax{5}\d+\z/}
   end
-end if test_views
+end if DB.supports_view_listing?
